/*

	by Serge FANTINO - fantino@math.unice.fr
	
*/

#include <Debug.h>

#include "data_control.h"

KEntry::KEntry()
{
	fEnabled=true;
}

void KEntry::Register(KDataControl *dctrl)
{
	dctrl->AddEntry(this);
}

bool KEntry::is_enabled()
{
	return fEnabled;
}

void KEntry::set_enabled(bool enabled)
{
	fEnabled=enabled;
}
	
// KDataControl ------------------------------------

KDataControl::KDataControl()
{
	fEnabled=true;
}

void KDataControl::AddEntry(KEntry *entry)
{
	fEntries[entry->EntryName()]=entry;
	entry->fDCtrl=this;
}

void KDataControl::Register(KEntry *entry)
{
	AddEntry(entry);
}

KEntry *KDataControl::FindEntry(const char *name) const
{
	//return fEntries[name];
	map<const char*,KEntry *,ltstr>::const_iterator iter=fEntries.find(name);
	if (iter==fEntries.end())
		return NULL;
	else
		return iter->second;
}

status_t KDataControl::SetValue(const char *name,KEntry *entry,BMessage *value) const
{
	status_t err=entry->SetValue(value);
	if (err==B_OK) ValueChanged(name);
	return err;
}

status_t KDataControl::SetValue(const char *name,BMessage *value) const
{
	PRINT(("set %s value\n",name));
	KEntry *entry=FindEntry(name);
	if (entry!=NULL) {
		return SetValue(name,entry,value);
	} else return B_ERROR;
}

status_t KDataControl::GetValue(const char *name,BMessage *value) const
{
	KEntry *entry=FindEntry(name);
	if (entry!=NULL) {
		return entry->GetValue(value);
	} else return B_ERROR;
}
	
status_t KDataControl::SetValues(BMessage *values) const
{
	map<const char*,KEntry *,ltstr>::const_iterator iter=fEntries.begin();
	while (iter!=fEntries.end()) {
		SetValue(iter->first,iter->second,values);
		iter++;
	}
	return B_OK;
}

status_t KDataControl::SetValues(BMessage *values,bool all) const
{
	if (all) return SetValues(values);
	// set only message fields...(all==false)
	char *name;
	type_code type;
	int32 index=0;
	PRINT(("set all values in message\n"));
	while (values->GetInfo(B_ANY_TYPE,index,&name,&type)==B_OK) {
		PRINT(("set the %iieme value %s\n",index,name));
		SetValue(name,values);
		index++;
	}
	return B_OK;
}

status_t KDataControl::GetValues(BMessage *values) const
{
	map<const char*,KEntry *,ltstr>::const_iterator iter=fEntries.begin();
	while (iter!=fEntries.end()) {
		iter->second->GetValue(values);
		iter++;
	}
	return B_OK;
}

status_t KDataControl::GetValues(BMessage *values,bool all) const
{
	if (all) return GetValues(values);
	// get only message fields...
	char *name;
	type_code type;
	int32 index;
	BMessage fields(*values);
	while (fields.GetInfo(B_ANY_TYPE,index,&name,&type)==B_OK) {
		GetValue(name,values);
		index++;
	}
	return B_OK;
}

void KDataControl::SetEnabled(const char *name,bool enabled) const
{
	KEntry *entry=FindEntry(name);
	if (entry!=NULL) return SetEnabled(entry,enabled);
}
	
void KDataControl::SetEnabled(KEntry *entry,bool enabled) const
{
	entry->set_enabled(enabled);
}
	
bool KDataControl::IsEnabled(const char *name) const
{
	KEntry *entry=FindEntry(name);
	if (entry!=NULL) return IsEnabled(entry);
}	

bool KDataControl::IsEnabled(KEntry *entry) const
{
	return entry->is_enabled();
}

bool KDataControl::IsEnabled() const
{
	return fEnabled;
}

void KDataControl::SetEnabled(bool enabled)
{
	fEnabled=enabled;
}

// KTextEntry --------------------------------------------------

KTextEntry::KTextEntry(BRect frame, const char *name, 
         const char *label, const char *text, 
         float divider,
         BMessage *message, 
         uint32 resizingMode,
         uint32 flags):
KTextControl(frame,name,label,text,divider,message,resizingMode,flags)
{
	fDCtrl=NULL;
}
/*
void KTextEntry::Register(KDataControl *dctrl)
{
	KEntry::Register(Name(),dctrl);
	fDCtrl=dctrl;
}
*/

const char *KTextEntry::EntryName() const {return Name();}

status_t KTextEntry::SetValue(BMessage *value)
{
	SetText(value->FindString(EntryName()));
	return B_OK;
}
	
status_t KTextEntry::GetValue(BMessage *value) const
{
	value->RemoveName(Name());
	return value->AddString(EntryName(),Text());
}

void KTextEntry::SetEnabled(bool enabled)
{
	if (Owner()!=NULL) {
		if (is_enabled()||!enabled) KTextControl::SetEnabled(enabled);
	} else {
		KTextControl::SetEnabled(enabled);
	}
}

// KCheckBoxEntry --------------------------------------------------

KCheckBoxEntry::KCheckBoxEntry(BRect frame, const char *name, 
         const char *label,
         BMessage *message, 
         uint32 resizingMode,
         uint32 flags):
BCheckBox(frame,name,label,message,resizingMode,flags)
{
}

const char *KCheckBoxEntry::EntryName() const {return Name();}

status_t KCheckBoxEntry::SetValue(BMessage *value)
{
	bool val;
	status_t err=value->FindBool(EntryName(),&val);
	if (err==B_OK) BCheckBox::SetValue(val?B_CONTROL_ON:B_CONTROL_OFF);
	return err;
}
	
status_t KCheckBoxEntry::GetValue(BMessage *value) const
{
	value->RemoveName(EntryName());
	return value->AddBool(EntryName(),Value()==B_CONTROL_ON);
}

// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


KEditView::KEditView(BRect frame,const char *name,float divider):
KPackView(frame,name,B_FOLLOW_ALL,B_NAVIGABLE_JUMP)
{
	fDivider=divider;
	//fEnabled=true;
}

//bool KEditView::IsEnabled() {return fEnabled;}

void KEditView::SetEnabled(bool enabled) 
{
	KDataControl::SetEnabled(enabled);
	BView *child=ChildAt(0);
	while (child!=NULL) {
		BControl *ctrl=dynamic_cast<BControl *>(child);
		if (ctrl!=NULL) ctrl->SetEnabled(enabled);
		child=child->NextSibling();
	}	
}

void KEditView::SetEnabled(KEntry *entry,bool enabled) 
{
	KDataControl::SetEnabled(entry);
	BControl *ctrl=dynamic_cast<BControl *>(entry);
	if (ctrl!=NULL) ctrl->SetEnabled(enabled);
}











