//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@math.unice.fr
//
//		Copyright 1998-1999, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include <stdio.h>

#include <AppKit.h>
#include <InterfaceKit.h>
#include <StorageKit.h>
#include <SupportKit.h>

#include "kool_kit.h"

/*
filter_result quit_requested_filter(BMessage *message, 
         BHandler **target, 
         BMessageFilter *messageFilter)
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return B_SKIP_MESSAGE;
}
*/

// rather than subclassing a window, I just use a Message Filter.
filter_result generic_filter(BMessage *message, 
         BHandler **target, 
         BMessageFilter *messageFilter)
{
	switch (message->what) {
		case B_ABOUT_REQUESTED:
			(new BAlert("About","Kool lib demo app\n\nby Serge Fantino","OK"))->Go(NULL);
			return B_SKIP_MESSAGE;
		case 'kTIP':
			tips_manager()->Enable(!tips_manager()->IsEnabled());
			void *source;
			if (message->FindPointer("source",&source)==B_OK) {
				BMenuItem *item=(BMenuItem *)source;
				item->SetMarked(tips_manager()->IsEnabled());
			}
			return B_SKIP_MESSAGE;
		default:
			return B_DISPATCH_MESSAGE;
	}
}

filter_result debug_filter(BMessage *message, 
         BHandler **target, 
         BMessageFilter *messageFilter)
{
	message->PrintToStream();
}

#define K_FLAG B_WILL_DRAW|B_FULL_UPDATE_ON_RESIZE

class KDemoWindow : public KAppWindow {
public:
	KDemoWindow();
	
	virtual void ReadyToPrepare();
	
	void MessageReceived(BMessage *message);
	
};

KDemoWindow::KDemoWindow():KAppWindow(BRect(50,50,500,500),"Layout Manager lib Test",B_TITLED_WINDOW,B_ASYNCHRONOUS_CONTROLS)
{
	// set up menu bar
	BMenu *mn;
	BMenuItem *item;
	// --- app menu
	mn=new BMenu("_app_menu_");
	mn->AddItem(new BMenuItem("About",new BMessage(B_ABOUT_REQUESTED),'A'));
	AddApplicationMenu(mn);
	// --- file menu
	mn=new BMenu("Kool Lib Demo");
	mn->AddItem(item=new BMenuItem("Show Tips",new BMessage('kTIP')));
	item->SetMarked(tips_manager()->IsEnabled());
	mn->AddSeparatorItem();
	mn->AddItem(new BMenuItem("New Window",new BMessage('kNEW')));
	AddFileMenu(mn);// get QUIT added automaticly
}

void KDemoWindow::ReadyToPrepare()
{
	BView *pp=NULL;
	// start packing child views
	// --- add a top button
	PackChild(new KSeparatorView(B_HORIZONTAL,""));
	PackChild(pp=new BButton(BRect(0,0,100,10),"up","upper side",NULL,0,K_FLAG),K_TOP,K_LEFT,K_FILL_X,BPoint(4,0));
	add_tips("this a resizable button anchored to top",pp);
	// --- split the view
	MainPackView()->Split(K_SPLITER);
	// --- add another button at top
	PackChild(pp=new BButton(BRect(0,0,100,10),"up2","upper side2",NULL,0,K_FLAG),K_TOP,K_LEFT,K_FILL_X);
	add_tips("this is another resizable button anchored to top",pp);
	// --- split again
	MainPackView()->Split(K_SPLITER);
	// --- add a bottom button
	PackChild(pp=new BButton(BRect(0,0,100,10),"bottom","bottom",NULL,0,K_FLAG),K_BOTTOM,K_LEFT,K_FILL_X);
	add_tips("this is a bottom button",pp);
	// --- add a left button
	PackChild(pp=new BButton(BRect(0,0,10,10),"left","left side",NULL,0,K_FLAG),K_LEFT,K_TOP,K_FILL_Y);
	add_tips("this is a left button",pp);
	// --- split after the left button - so it should be a vertical split...
	MainPackView()->Split();
	// --- preparing a tab view
	KTabView *tv=new KTabView(BRect(0,0,50,50),"tab_view");
	// --- --- ... using pack array
	KPack *pack_list2[]={
			new KPack(new BButton(BRect(0,0,200,10),"1","first",NULL,0,K_FLAG),K_TOP,K_LEFT,K_FILL_X),
			new KPack(new BButton(BRect(0,0,100,10),"2","second",NULL,0,K_FLAG),K_TOP,K_LEFT,K_FILL_X),
			new KPack(new BButton(BRect(0,0,100,10),"3","three",NULL,0,K_FLAG),K_TOP,K_LEFT,K_FILL_X),
			new KPack(new BButton(BRect(0,0,100,10),"4","four",NULL,0,K_FLAG),K_TOP,K_LEFT,K_FILL_X),
			NULL
	};
	KPack *pack_list3[]={
			new KPack(new KTextControl(BRect(0,0,100,10),"text1","some too long label","some odd text",130,NULL),K_TOP,K_LEFT,K_FILL_X,BPoint(0,5)),
			new KPack(new KTextControl(BRect(0,0,100,10),"text2","some very very too long label","some odd text",130,NULL),K_TOP,K_LEFT,K_FILL_X),
			new KPack(new KTextControl(BRect(0,0,100,10),"text3","some label","some odd text",130,NULL),K_TOP,K_LEFT,K_FILL_X,BPoint(0,5)),
			new KPack(new KTextControl(BRect(0,0,100,10),"text3","some label, yes...","some odd text",130,NULL),K_TOP,K_LEFT,K_FILL_X),
			NULL
	};
	// --- --- ... a tab with a scroll view
	BTextView *txt=new BTextView(BRect(0,0,50,50),"text_view",BRect(0,0,1000,50),B_FOLLOW_ALL,B_WILL_DRAW);
	txt->MakeResizable(false);
	KPack *pack_list4[]={
			new KPack(new BScrollView("scroll_view",txt,B_FOLLOW_ALL,0,false,true),K_FULL,K_LEFT,K_FILL_BOTH),
			NULL
	};
	// --- --- ... by hand
	KPackView *temp_pack=new KPackView(BRect(0,0,100,100),"tab one",B_FOLLOW_ALL,0);
	BRadioButton *rb=NULL;
	rb=new BRadioButton(BRect(0,0,10,10),"radio1","BeOS",new BMessage('xxxx'));
	add_tips("the only one",rb);
	temp_pack->PackChild(rb,K_TOP,K_LEFT);
	rb=new BRadioButton(BRect(0,0,10,10),"radio2","MacOS Z",new BMessage('xxxx'));
	temp_pack->PackChild(rb,K_TOP,K_LEFT);
	rb=new BRadioButton(BRect(0,0,10,10),"radio3","Win99",new BMessage('xxxx'));
	temp_pack->PackChild(rb,K_TOP,K_LEFT);
	rb=new BRadioButton(BRect(0,0,10,10),"radio4","Linux",new BMessage('xxxx'));
	temp_pack->PackChild(rb,K_TOP,K_LEFT);
	KLabelView *label=new KLabelView(BRect(0,0,200,10),"left","left marging label");
	label->SetAlignment(B_ALIGN_LEFT);
	temp_pack->PackChild(label,K_TOP,K_LEFT,K_FILL_X);
	label=new KLabelView(BRect(0,0,200,10),"center","center marging label");
	label->SetAlignment(B_ALIGN_CENTER);
	temp_pack->PackChild(label,K_TOP,K_LEFT,K_FILL_X);
	label=new KLabelView(BRect(0,0,200,5),"right","right marging label");
	label->SetAlignment(B_ALIGN_RIGHT);
	temp_pack->PackChild(label,K_TOP,K_LEFT,K_FILL_X);
	label=new KLabelView(BRect(0,0,200,10),"comment","<<when there's not enougth room, left alignment is use");
	label->SetAlignment(B_ALIGN_LEFT);
	temp_pack->PackChild(label,K_TOP,K_LEFT,K_FILL_X);
	add_tips("there's a problem here cause radio buttons are added after the handler is set...",temp_pack);
	// --- --- add the tabs
	//tv->AddTab(new BView(BRect(0,0,100,100),"test resizing",B_FOLLOW_ALL,0));
	tv->AddTab(temp_pack);
	tv->AddTab(new KPackView("tab two",pack_list3));
	tv->AddTab(new KPackView("tab zero",pack_list4));
	// --- --- build an embeded tab
	KTabView *sub_tv=new KTabView(BRect(0,0,100,100),"an embeded tab");
	tv->AddTab(sub_tv);
	sub_tv->AddTab(new BBox(BRect(0,0,200,200),"another very long tab name",B_FOLLOW_ALL,B_WILL_DRAW|B_FULL_UPDATE_ON_RESIZE));
	sub_tv->AddTab(new BButton(BRect(0,0,200,200),"some very long tab name","essai",new BMessage(),B_FOLLOW_ALL,B_WILL_DRAW|B_FULL_UPDATE_ON_RESIZE));
	sub_tv->AddTab(new BBox(BRect(0,0,400,200),"ok, but it's the last, hum ?",B_FOLLOW_ALL,B_WILL_DRAW|B_FULL_UPDATE_ON_RESIZE));
	// --- final packing
	PackChild(new BButton(BRect(0,0,100,10),"1","first",NULL,0,K_FLAG),K_TOP,K_LEFT,K_FILL_X);
	PackChild(new BButton(BRect(0,0,100,10),"2","second",NULL,0,K_FLAG),K_TOP,K_LEFT,K_FILL_X);
	PackChild(new BTextControl(BRect(0,0,110,10),"text1","some too long label","some odd text",NULL),K_TOP,K_LEFT,K_FILL_X);
	BMenu *test=new BPopUpMenu("some menu");
	test->AddItem(new BMenuItem("BeOS",new BMessage('xxxx')));
	test->AddItem(new BMenuItem("MacOS",new BMessage('xxxx')));
	test->AddItem(new BMenuItem("WinOS",new BMessage('xxxx')));
	test->AddItem(new BMenuItem("Linux",new BMessage('xxxx')));
	BMenuField *xx=new BMenuField(BRect(0,0,100,10),"menu","target OS:",test);
	xx->SetDivider(100.0);
	PackChild(xx,K_TOP,K_LEFT);
	add_tips("there's a problem with popup menu...",xx);
	PackChild(pp=new BTextControl(BRect(0,0,100,10),"text2","some very very too long label","some odd text",NULL),K_TOP,K_LEFT);
	add_tips("c'est un autre message a la con",pp);
	add_tips("did you notice who tips are often useless ?",tv);
	// dual pane
	PackChild(new KDualPaneView(BRect(0,0,100,100),"dual",tv,new BButton(BRect(0,0,10,10),"3","a resizable one",NULL,0,K_FLAG),K_LEFT),K_FULL,K_TOP,K_FILL_BOTH);
	// Load the readme.txt
	app_info info;
	be_app->GetAppInfo(&info);
	BEntry app_entry(&info.ref);
	BDirectory app_dir;
	app_entry.GetParent(&app_dir);
	BPath path(&app_dir,"../readme.txt");
	BFile read_me(path.Path(),B_OPEN);
	puts(path.Path());
	if (read_me.InitCheck()==B_NO_ERROR) {
		off_t size;
		read_me.GetSize(&size);
		txt->SetText(&read_me,0,size);
	} else
		txt->SetText("there 's supposed to be a readme file distributed with the lib...");
	// ----
}

void KDemoWindow::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case B_ABOUT_REQUESTED:
			(new BAlert("About","Kool lib demo app\n\nby Serge Fantino","OK"))->Go(NULL);
			break;
		case 'kTIP':{
			tips_manager()->Enable(!tips_manager()->IsEnabled());
			void *source;
			if (message->FindPointer("source",&source)==B_OK) {
				BMenuItem *item=(BMenuItem *)source;
				item->SetMarked(tips_manager()->IsEnabled());
			}
		} break;
		case 'kNEW':{
			KAppWindow *win=new KDemoWindow();
			win->AddToApp();
		} break;
		default:
			KAppWindow::MessageReceived(message);
	}
}

main() {

	KApp *app=new KApp("application/x-vnd.kmsf-KooLlibDemo");
	// ----
	KAppWindow *win=new KDemoWindow();
/*
	win->Lock();
	//win->AddCommonFilter(new BMessageFilter(B_QUIT_REQUESTED,quit_requested_filter));
	win->AddCommonFilter(new BMessageFilter(B_ABOUT_REQUESTED,generic_filter));
	win->AddCommonFilter(new BMessageFilter('kTIP',generic_filter));
	//win->AddCommonFilter(new BMessageFilter(B_PULSE,debug_filter));
	win->Unlock();
*/
	app->AddWindow(win);//win->Show();
	// ----	
	app->Run();
	// ----
	//delete tips;
	delete app;
	
}