/*

	by Serge FANTINO - fantino@math.unice.fr
	
*/

#ifndef DATA_CONTROL_H
#define DATA_CONTROL_H

#include <AppKit.h>
#include <InterfaceKit.h>

#include <map.h>

#include "text_control.h"
#include "pack_view.h"

class KDataControl;

class KEntry {
public:

	KEntry();

	void Register(KDataControl *dctrl);

	virtual const char *EntryName() const=0;
	virtual status_t SetValue(BMessage *value)=0;
	virtual status_t GetValue(BMessage *value) const=0;

protected:

	friend KDataControl;
	
	inline KDataControl *Owner() {return fDCtrl;};
	
	bool is_enabled();
	virtual void set_enabled(bool enabled);
	
private:
	
	KDataControl *fDCtrl;
	bool	fEnabled;
	
};

class KDataControl {
public:

	KDataControl();

protected:

	friend KEntry;
	virtual void AddEntry(KEntry *entry);
	
public:

	void Register(KEntry *entry);

	KEntry *FindEntry(const char *name) const;
	
	status_t SetValue(const char *name,BMessage *value) const;
	status_t GetValue(const char *name,BMessage *value) const;
	
	void SetEnabled(const char *name,bool enabled) const;
	void SetEnabled(KEntry *entry,bool enabled) const;
	bool IsEnabled(const char *name) const;
	bool IsEnabled(KEntry *entry) const;
	
	virtual void ValueChanged(const char *name) const {};
	
	status_t SetValues(BMessage *values) const;// all==true
	status_t SetValues(BMessage *values,bool all) const;
	status_t GetValues(BMessage *values) const;// all==true
	status_t GetValues(BMessage *values,bool all) const;
	
	bool IsEnabled() const;
	virtual void SetEnabled(bool enabled);
	
protected:
	status_t SetValue(const char *name,KEntry *entry,BMessage *value) const;
	
private:

	// entry are stored in a map, using entry name as key

	struct ltstr 
	{ 
 		bool operator()(const char* s1, const char* s2) const 
 		{ 
   			return strcmp(s1, s2) < 0; 
  		} 
	};

	map<const char*,KEntry *,ltstr> fEntries; 	
	
	// ---
	
	bool	fEnabled;
};

// GUI part

class KTextEntry : public KEntry, public KTextControl {
public:

	KTextEntry(BRect frame, const char *name, 
         const char *label, const char *text, 
         float divider=0,
         BMessage *message=NULL, 
         uint32 resizingMode=B_FOLLOW_ALL,
         uint32 flags=B_WILL_DRAW);
         
  	//virtual void Register(KDataControl *dctrl);
  	//KDataControl *DataControl();

	virtual const char *EntryName() const;

	virtual status_t SetValue(BMessage *value);
	virtual status_t GetValue(BMessage *value) const;
	
	virtual void SetEnabled(bool enabled);

private:

	KDataControl *fDCtrl;
};

class KCheckBoxEntry : public KEntry, public BCheckBox {
public:

	KCheckBoxEntry(BRect frame, const char *name, 
         	 const char *label,
        	 BMessage *message=NULL, 
        	 uint32 resizingMode=B_FOLLOW_ALL,
        	 uint32 flags=B_WILL_DRAW);
        	 
	virtual const char *EntryName() const;
	
	virtual status_t SetValue(BMessage *value);
	virtual status_t GetValue(BMessage *value) const;
	
};

class KEditView : public KPackView, public BInvoker, public KDataControl {
public:

	KEditView(BRect frame,const char *name,float divider);
	
	//bool IsEnabled();
	void SetEnabled(bool enabled);
	void SetEnabled(KEntry *entry,bool enabled);
	
	//virtual void AllAttached();
	
protected:

	float	fDivider;
	//bool	fEnabled;
};

#endif




